/* nobsdes5xmpl.do -- hst 23.04.2020 / 30.06.2020 / 21.02.2022 / 28.02.2023 / 13.11.2023 */

/* ******************************************************** */
/*                                                          */
/*                   Create artificial data for             */
/*                                                          */
/*                    D E S C R I P T I V E S               */
/*                                                          */
/*    11 IDs:             1-  11                            */
/*    10 years:        2001-2010                            */
/*     3 countries:  17, 39, 400                            */
/*                                                          */
/* ******************************************************** */
version 15

clear
set obs 301
generate hvcountry = int((_n-1)/100)+1
generate hv     = _n-100*(hvcountry-1)
generate id     = int((hv-1)/10)+1
generate year   = 2000+hv-10*(id-1)
drop hv

generate str2 country = "BE" if hvcountry==1
replace       country = "CH" if hvcountry==2
replace       country = "US" if hvcountry==3
replace       country = "GB" if hvcountry==4
drop hvcountry

set seed 4869382
generate x = runiform()
label variable x "Default right hand side variable"

* Creating missing values
replace x = . if year==2002 & id > 2        /* Yearly dummies are often used as 
							catch all dummies. However, a dummy for the year  
							2002 would identify IDs 1 and 2. The coefficient for  
							the 2002 dummy thus causes a disclosure problem.  */
replace x = . if _n==301

* Creating problematic IDs
replace id = 11 if _n==301      /* ID must be ignored, since x==. (see below) */

* Dummy if id==1
generate id1 = id==1            /* Publishing information on specific firms is 
							  	   not permitted.                             */

* Creating problematic dummies
* Dummy id/year/country: 1/2005/BE, 6/2007/BE. 
* This dummy is 1 for just two distinct IDs. Therefore, it must not be published.
generate dum_2ids   = _n==5|_n==57    
label variable dum_2ids "Dummy for two distinct IDs"

* Dummy id/year/country: 1/2005/BE, 6/2007/BE, 7/2003/BE, 8/2008/CH, 10/2009/BE. 
* This dummy is 1 for five distinct IDs. It may be published.
generate dum_5ids = _n==5|_n==57|_n==63|_n==99|_n==178 
label variable dum_5ids "Dummy for five distinct IDs"

* Creating data for percentiles option (xpc={1,...,99})
set seed 9385542
generate u = runiform()
sort u
drop u
generate int  xpc   = _n if _n<=99
generate byte pcdum = (xpc>50 & xpc<=60)                 /* problematic range */
* Only four distinct IDs in intervall (p50,p60]: 4, 5, 8, 10 
replace id =  4 if pcdum & (id==1|id==3|id==6|id==7)
replace id =  3 if xpc==67|xpc==74|xpc==82|xpc==87
replace id =  2 if xpc==17|xpc==33|xpc==93|xpc==94|xpc==99
replace id =  6 if xpc==77|xpc==78
replace id = 10 if xpc== 1|xpc== 7
replace id =  7 if id==5 & xpc!=8 & xpc!=57 & xpc!=.
replace id =  1 if id==8 & xpc!=58 & xpc!=.
* Trigger 1: If set, less than 15 percent outside (p50,p60] 
replace id =  9 if xpc==40  
* Trigger 2: If set, only four distinct IDs within (p90,max]
replace id =  3 if xpc==92 
label variable xpc "Variable for non deterministic categories"

* Creating variables for dominance check
generate byte xfordom = 5 if id==1
replace       xfordom = 4 if id==2
replace       xfordom =-4 if id==2 & country=="US"
replace       xfordom = 1 if id==3
replace       xfordom =10 if id==3 & country=="CH" & year>=2009
label variable xfordom "Variable for dominance"

* Creating variables for the treatment of zeros
generate xwithzeros = runiform()
replace  xwithzeros = 0 if id==3 & year<2009
label variable xwithzeros "Variable with zeros for id = 3 and year < 2009"

cls
* global endmark "$$"
version 17

/* ******************************************************** */
/*                                                          */
/*         N O B S D E S 5  without   O P T I O N S         */
/*                                                          */
/*              There are 11 distinct IDs but               */
/*              x is set to missing for id==11              */
/*                                                          */
/* ******************************************************** */

nobsdes5 id x      /* reports the number of distinct values */
 /* of the identifier for nonmissing values of the variable */
nobsdes5 id        /* reports the number of distinct IDs    */

nobsdes5 id x xpc  /* varlist possible since version 2.0.0  */

/* ******************************************************** */
/*                                                          */
/*       H o w   to   u s e   the   B Y   O P T I O N       */
/*                                                          */
/*     In 2002 x is non missing only for id==1 and id==2    */
/*                                                          */
/*     To see whether there is a problem at all,            */
/*     use for large tables in the first step the           */
/*     n o t a b   O P T I O N.                             */
/*                                                          */
/* ******************************************************** */

nobsdes5 id x, by(year) notab 

/* ******************************************************** */
/*                                                          */
/*     Now you know there is a problem, but you do not      */
/*     know the cause. Thus, in the second step, do not     */
/*     use   n o t a b.                                     */
/*                                                          */
/* ******************************************************** */

nobsdes5 id x, by(year)

/* ******************************************************** */
/*                                                          */
/*     Now you know that the problem is the year 2002. In   */
/*     this case, there seems to be a simple solution to    */
/*     receive the output.                                  */
/*                                                          */
/* ******************************************************** */

capture noisily table year if year!=2002, statistic(total x)

/* ******************************************************** */
/*                                                          */
/*     However, if you drop just one row, the hidden        */
/*     content may be inferred by the disclosed rows        */
/*     and the total. You can investigate this using        */
/*     the   g e n   O P T I O N.                           */
/*                                                          */
/* ******************************************************** */

nobsdes5 id x, by(year) notab generate replace
egen hvnobs = sum(tg_id_x),   by(year)
egen hvcr2  = mean(cr2_id_x), by(year)
nobsdes5 id x if (hvnobs<=4 | (hvcr2>85 & hvcr2<=100))

/* ******************************************************** */
/*                                                          */
/*     Thus, you must hide at least two rows, or, better    */
/*     yet, aggregate.                                      */
/*                                                          */
/* ******************************************************** */

quietly generate int bisyear = 2002 if (year==2001|year==2002)
quietly replace  bisyear = 2004 if (year==2003|year==2004)
quietly replace  bisyear = 2006 if (year==2005|year==2006)
quietly replace  bisyear = 2008 if (year==2007|year==2008)
quietly replace  bisyear = 2010 if (year==2009|year==2010)

nobsdes5 id x, by(bisyear)
table bisyear, statistic(total x) nototals

/* ******************************************************** */
/*                                                          */
/*                        Z E R O S                         */
/*                                                          */
/*     In the case of continuous variables zeros can occur  */
/*     for a certain entity always, sometimes, or never.    */
/*     If zeros always occur, then that particular entity   */
/*     never conducts the respective type of business.      */
/*     Some readers may be aware of this. These entities    */
/*     are not counted. If zeros instead occur              */
/*     infrequently, this could mean that this entity       */
/*     generally conduct the type of business captured      */
/*     by this variable, but sometimes does not. It is      */
/*     presumed that competitors are unaware of this.       */
/*     These entities could be counted. Since it is         */
/*     difficult to define a threshold of fractions of      */
/*     zeros above which an entity is not counted, we       */
/*     always treat zeros as missing values for pragmatic   */
/*     reasons if output control is carried out for         */
/*     descriptive statistics. Use the                      */
/*     m i s s   O P T I O N.                               */
/*                                                          */
/*     In the case of dummy variables and categorical       */
/*     variables, a zero has a different meaning. Here,     */
/*     zeros are valid observations.                        */
/*                                                          */
/* ******************************************************** */

nobsdes5 id xwithzeros, by(year) 
nobsdes5 id xwithzeros, by(year) miss(0)

/* ******************************************************** */
/*                                                          */
/*                    D O M I N A N C E                     */
/*                                                          */
/*     There are five different IDs with values for         */
/*     xfordom in each year.                                */
/*                                                          */
/* ******************************************************** */

table id country if xfordom!=., statistic(total xfordom) 

/* ******************************************************** */
/*                                                          */
/*     Assume that id==2 knows that there are two other     */
/*     IDs and that id==1 is much larger than id==3. If     */
/*     you publish the sum, id==2 can estimate the value    */
/*     of xfordom for id==1.                                */
/*     For country=="BE" it is near 112-44 = 68. That is    */
/*     not too far from 55.                                 */
/*     For country=="US" it is near  31- 9 = 22 and thus    */
/*     far from 70.                                         */
/*     For country=="CH" it is near 128-48 = 80 and thus    */
/*     far from 50.                                         */
/*                                                          */
/* ******************************************************** */

nobsdes5 id xfordom, by(country)

*  The data is now sorted

des

/* ******************************************************** */
/*                                                          */
/*            M A X I M U M   and   M I N I M U M           */
/*                                                          */
/*     M A X I M U M   and   M I N I M U M   are single     */
/*     observations. Therefore, they must not be shown.     */
/*     If necessary, the RDSC allows the publication of     */
/*     approximate values calculated as averages of a       */
/*     sufficient number of observations (top coding).      */
/*     For maximum and minimum, at least five distinct      */
/*     entities must be covered and the share of the two    */
/*     largest entities ones must not exceed 85 percent     */
/*     of the total (dominance criterion).                  */
/*                                                          */
/*     Please use the .ado file  maxrdsc  to address        */
/*     this problem. If a table or cross-tabulation         */
/*     is desired, use nobsdes5 after completing some       */
/*     preparatory steps.                                   */
/*                                                          */
/* ******************************************************** */

maxrdsc id x, min(12) max(12)     

/* ******************************************************** */
/*                                                          */
/*                    Q U A N T I L E S                     */
/*                                                          */
/*     Quantiles must be specified in ascending order.      */
/*     The RDSC treats quantiles as defining categories.    */
/*     The number of distinct identifiers between the       */
/*     minimum and the lowest percentile as well between    */
/*     the highest percentile and the maximum and between   */
/*     all other consecutive quantiles have to be larger    */
/*     four and the dominance criterion has to be obeyed.   */
/*                                                          */
/*     Most often, quantiles are used to describe the       */
/*     shape of a distribution and tied values are no       */
/*     issue then. If quantiles serve to create groups      */
/*     instead, ties may be an issue.                       */
/*                                                          */
/* ******************************************************** */

nobsdes5 id xpc if year==2010, pctile(5 50 90)
nobsdes5 id xpc, pctile(5 50 90) by(year) notab
nobsdes5 id xpc, pctile(10 20 30 40 50 60 70 80 90) 

/* ******************************************************** */
/*                                                          */
/*               C R E A T I N G   G R A P H S              */
/*                                                          */
/*     If you want to create a graph you must document      */
/*     the fact that the graph is based on sufficient       */
/*     observations and that no two identifiers dominate.   */
/*     You must use   n o b s d e s 5   before you create   */
/*     the graph. We recommend to save the graph as a       */ 
/*     .png file.                                           */
/*                                                          */
/* ******************************************************** */

nobsdes5 id x, by(year country) notab
nobsdes5 id x, by(year country) 
egen meanx  = mean(x) if year!=2002, by(year country)
egen tgcountry = tag(year country)
gsort -tgcountry year country
scatter meanx year if country=="BE" & tgcountry
* graph save graphname.png

/* ******************************************************** */
/*                                                          */
/*           C R E A T I N G   H I S T O G R A M S          */
/*                                                          */
/*     If you want to create a histogram you must           */
/*     document the fact that the graph is based on         */
/*     sufficient observations and that no two IDs          */
/*     dominate. You must use   n o b s d e s 5             */
/*     before you create the histogram. We recommend        */
/*     saving the histogram as a .png file.                 */
/*                                                          */
/* ******************************************************** */

nobsdes5 id xpc, histogram(10)              /* e.g. 10 bins */
* This differs from the height of the bins
quietly histogram xpc, freq bin(10) addlabels

* varlist possible since version 2.0.0  
nobsdes5 id x xpc, histogram(25 10)  

* A good alternative to a histogram is a kernel density. 
* You have to show, that the kernel density is based on at 
* least five distinct ids.

kdensity xpc
nobsdes5 id xpc

/* ******************************************************** */
/*                                                          */
/*              A G G R E G A T I N G   D A T A             */
/*                                                          */
/*     If you want to aggregate data for further use,       */
/*     you must document the fact that the aggregate        */
/*     consists of a sufficient number of observations      */
/*     and that no two identifiers dominate. You must       */
/*     use   n o b s d e s 5   before you collapse the      */
/*     data.                                                */
/*                                                          */
/* ******************************************************** */
