/* nobsreg5xmpl.do -- hst 19.03.2020 */

clear

/* ******************************************************** */
/*                                                          */
/*                   Create artificial data for             */
/*                                                          */
/*                     R E G R E S S I O N S                */
/*                                                          */
/*    11 IDs:             1-  11                            */
/*    10 years:        2001-2010                            */
/*     4 countries:  BE, CH, US, GB                         */
/*                                                          */
/* ******************************************************** */

set obs 301
generate hvcountry = int((_n-1)/100)+1
generate hv        = _n-100*(hvcountry-1)
generate byte id   = int((hv-1)/10)+1
generate int year  = 2000+hv-10*(id-1)
drop hv

generate str2 country = "BE" if hvcountry==1
replace       country = "CH" if hvcountry==2
replace       country = "US" if hvcountry==3
replace       country = "GB" if hvcountry==4
drop hvcountry

set seed 4869382
generate y = runiform()
generate x = runiform()
generate z = runiform()

* Creating missing values
replace x = . if year==2002 & id > 2        /* Yearly dummies are often used as 
							catch all dummies. However, a dummy for the year  
							2002 would identify IDs 1 and 2. The coefficient for  
							the 2002 dummy thus causes a disclosure problem.  */
replace x = . if _n==301

* Creating problematic IDs
replace id = 11 if _n==301      /* ID must be ignored, since x==. (see below) */

* Dummy if id==1
generate id1 = id==1            /* Publishing information on specific firms is 
							  	   not permitted.                             */

* Creating problematic dummies
* Dummy id/year/country: 1/2005/BE, 6/2007/BE. 
* This dummy is 1 for just two distinct IDs. Therefore, it must not be published.
generate dum_2ids   = _n==5|_n==57    

* Dummy id/year/country: 1/2005/BE, 6/2007/BE, 7/2003/BE, 8/2008/CH, 10/2009/BE. 
* This dummy is 1 for five distinct IDs. It may be published.
generate dum_5ids = _n==5|_n==57|_n==63|_n==99|_n==178 
*
replace z = 0 if id!=1  /* Since z has values different from zero for one ID
                           only, this ID is identified by z. Thus, no regression
						   coeffcient for z must be published. nobsreg creates a
						   dichotomous auxiliary variable that is 0 if z = 0 and
						   1 if z <> 0. If z = 1 for only one or two IDs a 
						   disclosure problem is reported.                    */

/* Later on there is an example using two different identifiers. This occurs  
   e.g. if the data includes domestic firms as well as their affiliates abroad. 
   Then confidentiality has to be assured for both, parent institutes and 
   subsidiaries.                                                              */
generate byte idtest = id                   
replace idtest = 5 if year==2010  

cls

* global endmark "$$"     /* This puts $$ after each nobsreg output. Using    */
                          /* nobsclean.do, you can erase this output.         */
						  
/* ******************************************************** */
/*                                                          */
/*               H o w   to   u s e   nobsreg               */
/*      w i t h   and   w i t h o u t   global idlist       */
/*                                                          */
/* ******************************************************** */

*  Set global idlist. Do not forget "". 

global idlist "id"

regress y x z dum_2ids dum_5ids i.year
nobsreg5

*  It does not always suffice to investigate confidentiality for one identifier
*  only. For example, a sample may consist of two MFIs (id) with each having 
*  three subsidiaries (idsub) abroad. Confidentiality must then be assured for
*  both the parent institutions and their subsidiaries. nobsreg loops through
*  the idlist. 

*  global idlist is preferable if you run several regressions using
*  the same identifiers. 

global idlist "id idtest"

regress y x z dum_2ids dum_5ids i.year
nobsreg5

*  Now try the following

nobsreg5 id

*  and again

nobsreg5

/* ******************************************************** */
/*                                                          */
/*          H o w   to   u s e   nobsreg   w i t h          */
/*      "p a r t i c u l a r"   r e g r e s s i o n s       */
/*                                                          */
/* ******************************************************** */

global idlist "id"

*  nobsreg can deal with the bootstrap command

bootstrap, reps(10): reg y x z dum_2ids dum_5ids i.year
nobsreg5

*  however, the last draw can be a bad one. Run an artificial 
*  OLS regression

quietly regress y x z dum_2ids dum_5ids i.year
nobsreg5

/* ******************************************************** */
/*                                                          */
/*          H o w   to   o b t a i n   r e s u l t s        */
/*                                                          */
/* ******************************************************** */

*  You are not allowed to show or take with you the results 
*  for the variables z, dum_2ids and 2002.year. However, you 
*  can quietly regress, invoke nobsreg5, store the results 
*  and drop problematic results. 

eststo: quietly reg y x z dum_2ids dum_5ids i.year
nobsreg5
esttab, drop(z dum_2ids 2001.year 2002.year)

*  In this example, nobsreg5 indicates a problem with the year 2002.
*  However, it does not provide the coefficient, nor does it show,
*  whether the problem refers to one or two units.

*  You may save the admissible output in a separate file

* esttab using ${path_log}/AllowedOutput.tex, drop(z dum_2ids 2001.year 2002.year)

*  Ask for the do-file so that you know your commands producing
*  this output, and provide the RDSC with the "usual" log-file
*  with the nobsreg5 output, so that the RDSC can see which
*  coefficients must not be disclosed (z dum_2ids 2001.year 2002.year).

/* ******************************************************** */
/*                                                          */
/*                V a r i a b l e l i s t s                 */
/*                                                          */
/* ******************************************************** */

*  It is possible to use locals and interactions as well as to uncomment 
*  variables. Up to three variables can be interacted at a time.

*  Countries are coded according to ISO 3166-2 i.e. as two letters.
*  However, factor-variables have to be numeric. Create numeric "countrynum".

sort country
egen int countrynum = group(country)

local dummies "dum_2ids dum_5ids"
regress y x z `dummies' /* z */ year##countrynum

*  A problem with  year  and the interaction between  year and country
*  will be reported because of the year 2002. There is no problem 
*  with  country.

nobsreg5 

*  Requirements for regional variables are stricter than for the others.
*  You must use the regiovar-option to mark the regional variable.
*  For the sake of simplicity, we assume "countrynum" to be a regional 
*  variable in this example. 

nobsreg5 , regio(countrynum)

*  The following line is not requested for disclosure control.
*  However, if you are concerned, e.g. with regard to data mining, type: 

mata: nobstable

*  Continuous variables can be interacted with a discrete variable, 
*  but you must use c.var. 

generate byte CH = country=="CH"
regress y x year##CH##c.z
nobsreg5

*  nobsreg5 can cope with up to three interactions

regress y x year##CH#c.z
nobsreg5
regress y x year#c.z##CH
nobsreg5
regress y x year#c.z#CH
nobsreg5

*  It is possible to use time-series operators

egen idcountry = group(id country)
xtset idcountry year, yearly

regress y x z dum_2ids dum_5ids id1 i.year l.z, nocons
nobsreg5

regress y x z L(1 2/3).z id1 i.year 
nobsreg5

*  To interact a lagged variable, it must be created. 
*  il.year does not work neither does year##l.x.

regress y z dum_2ids dum_5ids year##c.x
nobsreg5
